new;
library optmum,pgraph;

#include regGMM.src

X0 = 10;
a = 0.8;
b = 0.1;
sigma = 0.06;
h = 0.1;
Nobs = 1000;
sv = a|b|sigma;

/* Generate an Ornstein-Uhlenbeck process */

xt = zeros(Nobs,1);
xt[1] = x0;

k1 = exp(-a*h);
k2 = (1-exp(-2*a*h))/(2*a);

u = (sigma*sqrt(k2))*rndn(Nobs-1,1);
u = u+b*(1-k1);

i = 2;
do until i > Nobs;
  xt[i] = k1*xt[i-1]+u[i-1];
  i = i + 1;
endo;

proc ml(theta);
  local a,b,sigma,k1,k2,epsilon,logl;

  a = theta[1];
  b = theta[2];
  sigma = theta[3];

  k1 = exp(-a*h);
  k2 = sigma^2*(1-exp(-2*a*h))/(2*a);

  epsilon = xt[2:Nobs]-k1.*xt[1:Nobs-1]-b*(1-k1);

  logl = -0.5*ln(2*pi) - 0.5*ln(k2) - 0.5*epsilon^2/k2;

  retp(logl);
endp;

fn negLogL(theta) = -sumc(ml(theta));

proc gmm(theta);
  local a,b,sigma,k1,k2,epsilon,M;

  a = theta[1];
  b = theta[2];
  sigma = theta[3];

  k1 = exp(-a*h);
  k2 = sigma^2*(1-exp(-2*a*h))/(2*a);

  epsilon = xt[2:Nobs]-k1.*xt[1:Nobs-1]-b*(1-k1);

  M = epsilon~(epsilon^2-k2)~(epsilon.*xt[1:Nobs-1]);

  retp(M);
endp;

parnm = "a"|"b"|"sigma";

{theta1,fmin,grd,retcode} = optmum(&negLogL,sv);

{theta2,stderr,Mcov,Qmin} = regGMM(&gmm,sv,0,0);

print; print;
print "                 True values        TDML            GMM";
call printfmt(parnm~sv~theta1~theta2,0~1~1~1);

